;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;Wind Chimes by Greg Harder, 2024
;================================

ORG 4084

@Seed=4032			;Random Number Seed
@Keyboard=02BB			;ROM: Keyboard Scan
@Freq=403E			;16448 to 16462 Freq Table

	JR Chimes
@Bang				;16518, Bong Rate
	00
@EnvShape			;16519, Envelope Shape
	00

;================================
;Chimes
;================================
@Chimes
	LD A,(Bang)		;Get Bong Rate
	LD (Bong+1),A		;Set Bong Rate
	LD A,(EnvShape)		;Get Envelope Shape
	LD (Shape+1),A		;Set Envelope Shape
@Wait
	CALL Keyboard		;Scan keyboard
	INC L			;FF test
	JR NZ Wait		;Loop while key pressed
@ChimeLup
	CALL Keyboard		;Scan keyboard
	INC L			;FF test
	RET NZ			;Return to BASIC if Key pressed
	CALL Rnd3		;A=1 to 3
	CP 01			;RX=01
	JR Z SetRX
	CP 03			;RX=03
	JR Z SetRX
	LD A,05			;RX=05
@SetRX
	LD B,A
	CALL BangAGong		;Sound A with random freq.
	JR ChimeLup		;Loop until key pressed
;===============================
@BangAGong
	PUSH BC			;PUSH channel C1
	CALL Rnd6		;A=1 to 6
	POP BC			;POP channel C1
	LD HL,Freq		;Freq. table-2
	LD D,00			;Locate freq. in table
	LD E,A			;Get A,B or C chime
	AND A			;Clear CY
	RL E			;Double it
	ADD HL,DE		;HL=Freq. address
				;RX=C1 for A,B or C
	LD C,(HL)		;DX=Freq. coarse
	CALL Send2AY		;Set C1
	DEC B			;RX=C0 for A,B or C
	INC HL
	LD C,(HL)		;DX=Freq. fine
	CALL Send2AY		;Set C0
				;Sound new frequency
	LD B,0D			;RX=Envelope shape
@Shape
	LD C,00			;DX=Envelope, set at startup
	CALL Send2AY		;Sound chime
;===============================
;Random Ring Time
;===============================
	CALL Rnd6
@Bong
	LD H,00			;Bong Rate, set at startup
	BIT 0,A			;Odd/Even Test
	JR Z RingLup		;Jump if odd
	AND A			;Clear CY
	RR H			;H=H/2
@RingLup			;HL=Bong or Bong/2
	DEC HL
	LD A,H 
	OR L
	JR NZ RingLup
	RET
;===============================
; Send to AY
;  On entry B=RX  C=DX
;===============================
@Send2AY
	LD A,B			;Set RX
	OUT DF,A		;Send RX
	LD A,C			;Set DX
	OUT 0F,A		;Send DX
	RET
;===============================
;On exit A=1 to 3
;===============================
@Rnd3
	LD A,03
	LD (DivB+1),A
	JR Rnd
;===============================
;On exit A=1 to 6
;===============================
@Rnd6
	LD A,06
	LD (DivB+1),A
;===============================
; Random number generator
;===============================
@Rnd
	LD DE,(Seed)		;Get (Seed)
	LD H,E 			;H=Seed LO
	LD L,FD 		;L=253
	LD A,D 			;A=Seed HI
	OR A 			;Reset CY flag
	LD B,00 		;Clear B
	SBC HL,DE 		;Subtract seed from HL
	SBC A,B 		;Subtract CY from A
	SBC HL,DE  		;Subtract seed from HL again
	SBC A,B  		;Subtract CY from A again
	LD E,A 			;E=A
	LD D,B 			;Clear D
	SBC HL,DE		;Subtract DE from HL 
	JR NC +1		;If HL positive, set random number now
	INC HL 			;Otherwise increase HL first
	LD (Seed),HL		;Set (Seed) value
	LD A,L			;A=0 to 255
	AND 07			;A=0 to 7
@DivB
	LD B,06
@Mod
	SUB B
	JR NC Mod
	ADD A,B
	INC A			;A=1 to 6
;	LD C,A
;	LD B,00
	RET